//
// main.ino
//
// This software is intended to be run on a small Arduino (e.g., a Pro Mini),
// whose purpose is dedicated to talking to the RC system.  That is to say, it
// listens to the PPM signal from the RC receiver and produces a compatible
// single for the ESCs to control the motors.  It additionally talks to the main
// board via a trivial serial protocol.  In so doing, this software relieves the
// main board (and software running on it) of the task of interfacing with any
// of the RC system.
//
// Receiver
// --------
//
// The 8 channels (from the RC receiver) need to be combined and fed to two
// interrupts on the Arduino.  They need to be combined in such a way as to
// alternate pulses between the two interrupts (which are alternately used to
// read the pulses).  Like this:
//
//                  ch1  ch3  ch5  ch7  ch2  ch4  ch6  ch8
//                   |    |    |    |    |    |    |    |
//                   ▼    ▼    ▼    ▼    ▼    ▼    ▼    ▼
//                   ¯    ¯    ¯    ¯    ¯    ¯    ¯    ¯
//            ___    |    |    |    |    |    |    |    |     ___
//    GND ---|___|---+----+----+----+    +----+----+----+----|___|--- GND
//             R                    |    |                     R
//             ____________________ | __ | _____________________
//                                  |    |
//                           pin 2  o    o  pin 3
//                    (interrupt 0)        (interrupt 1)
//
// The two resistors in the circuit are pull-down resistors (so, say 100kΩ).
// Without them, the Arduino is unable to read the pulse wave at all.
//
// Note that your receiver may not send pulses sequentially, in "channel order".
// If this turns out to be the case, you will need to arrange for the pulses to
// wire up the channels in an order whereby the pulses alternatively alternately
// between the two interrupts and adjust the channel order below.
//
// Motor control
// -------------
//
// 


#include "config.h"
#include "receiver.h"
#include "motors.h"
#include "comms.h"


void setup()
{
	Receiver::setup();
	Motors::setup();
	Comms::setup();

	// led
	pinMode( LED_PIN, OUTPUT );
	digitalWrite( LED_PIN, LOW );
}


void flash_led()
{
	static bool on = false;
	on = !on;
	digitalWrite( LED_PIN, on? HIGH : LOW );
}


void loop()
{
	int channel_values[ NUM_CHANNELS ];

	while( true )
	{
		// have we read a whole set of channel values?
		if( Receiver::read_channels( channel_values ) )
		{
			// yes, flash led!
			flash_led();

			// send channel values to main board
			Comms::write_channels( channel_values );

			// set motor speeds
			Motors::set_values( channel_values );
		}

		// update motors
		Motors::update();
	}
}
